@@ -1,15 +1,21 @@
+import contextlib
 import json
+import logging
+import logging.handlers
 import os
-import shutil
-import subprocess
+from datetime import datetime, timezone
 from pathlib import Path
+from typing import Any, Generator
 from uuid import UUID
 
 import typer
+from chaoslib.control import load_global_controls
+from chaoslib.experiment import ensure_experiment_is_valid, run_experiment
+from chaoslib.loader import load_experiment
+from chaoslib.types import Journal, Schedule, Strategy
 
 from ..client import api_client
 from ..config import ensure_config_is_set, get_settings
-from ..config.types import get_settings_directory_path
 from ..format import format_as
 from ..log import console
 from ..types import FormatOption, Plan
@@ -52,37 +58,11 @@ def execute(
     """
     ensure_config_is_set()
 
-    settings_dir = get_settings_directory_path()
     p = load_plan(plan_id)
 
+    context = {}
     if not skip_context:
-        store_plan_context(p)
-
-    plan_dir = settings_dir / "plans" / str(p.id)
-    ctk_settings_file = str(plan_dir / "ctk.yaml")
-
-    args = [
-        shutil.which("chaos"),
-        "--no-version-check",
-        "--verbose",
-        "--log-file",
-        log_file,
-        "--log-file-level",
-        "debug",
-        "--settings",
-        ctk_settings_file,
-        "run",
-        "--journal-path",
-        str(result_file),
-    ]
-
-    for int_id in p.definition.integrations:
-        args.extend(
-            (
-                "--control-file",
-                str(plan_dir / "controls" / f"{str(int_id)}.json"),
-            )
-        )
+        context = store_plan_context(p)
 
     settings = get_settings()
 
@@ -104,34 +84,13 @@ def execute(
     experiment_id = p.definition.experiments[0]
     base_url = f"{settings.service.host}/api/v1/organization"
     base_url = f"{base_url}/{settings.organization.id}"
-
-    args.append(f"{base_url}/experiments/{experiment_id}/raw")
+    experiment_url = f"{base_url}/experiments/{experiment_id}/raw"
 
     with console.status("Executing..."):
-        return_code = 0
-        with subprocess.Popen(
-            args, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-        ) as proc:
-            try:
-                while True:
-                    try:
-                        proc.communicate(timeout=3)
-                    except subprocess.TimeoutExpired:
-                        pass
-
-                    retcode = proc.poll()
-                    if retcode is not None:
-                        return_code = retcode
-                        break
-            except KeyboardInterrupt:
-                # propagate the interruption to the chaostoolkit process
-                # for a clean exit
-                proc.terminate()
-                raise
-
-        if return_code:
-            console.print(f"failed to execute plan. see {log_file.absolute()}")
-            raise typer.Exit(code=return_code)
+        with reconfigure_chaostoolkit_logger(log_file):
+            journal = run_chaostoolkit(experiment_url, context)
+            result_file.absolute().write_text(json.dumps(journal, indent=2))
+            show_result_url(journal)
 
 
 ###############################################################################
@@ -154,15 +113,8 @@ def load_plan(plan_id: UUID) -> Plan:
             return Plan.parse_obj(r.json())
 
 
-def store_plan_context(plan: Plan) -> None:
-    settings_dir = get_settings_directory_path()
-
-    plan_dir = settings_dir / "plans" / str(plan.id)
-    plan_dir.mkdir(parents=True, exist_ok=True)
-
-    (plan_dir / "ctk.yaml").write_text(
-        "runtime:\n  rollbacks:\n    strategy: always\n"
-    )
+def store_plan_context(plan: Plan) -> dict[str, Any]:
+    global_controls = {}
 
     with console.status("Storing context..."):
         with api_client() as client:
@@ -170,9 +122,6 @@ def store_plan_context(plan: Plan) -> None:
                 r = client.get(f"/integrations/{int_id}/control")
                 control = r.json()
                 if control:
-                    ctrl_dir = plan_dir / "controls"
-                    ctrl_dir.mkdir(parents=True, exist_ok=True)
-
                     ctrl_name = control.pop("name")
                     provider = control.get("provider", {})
                     if "secrets" in provider and provider["secrets"] is None:
@@ -183,6 +132,86 @@ def store_plan_context(plan: Plan) -> None:
                     ):
                         del provider["arguments"]
                     ctrl = {ctrl_name: control}
-                    (ctrl_dir / f"{str(int_id)}.json").write_text(
-                        json.dumps(ctrl)
-                    )
+                    global_controls.update(ctrl)
+
+    return global_controls
+
+
+def run_chaostoolkit(experiment_url: str, context: dict[str, Any]) -> Journal:
+    logger = logging.getLogger("logzero_default")
+
+    logger.info("#" * 80)
+    logger.info(f"Starting Reliably experiment: {experiment_url}")
+
+    settings = {
+        "runtime": {
+            "hypothesis": {"strategy": "default"},
+            "rollbacks": {"strategy": "always"},
+        },
+        "controls": context,
+    }
+
+    load_global_controls(settings)
+    experiment = load_experiment(experiment_url, settings, verify_tls=True)
+    ensure_experiment_is_valid(experiment)
+
+    x_runtime = experiment.get("runtime")
+    if x_runtime:
+        settings["runtime"]["rollbacks"]["strategy"] = x_runtime.get(
+            "rollbacks", {}
+        ).get("strategy", "always")
+        settings["runtime"]["hypothesis"]["strategy"] = x_runtime.get(
+            "hypothesis", {}
+        ).get("strategy", "default")
+
+    schedule = Schedule(continuous_hypothesis_frequency=1.0, fail_fast=True)
+    experiment_vars = ({}, {})
+    ssh_strategy = Strategy.DEFAULT
+
+    journal = run_experiment(
+        experiment,
+        settings=settings,
+        strategy=ssh_strategy,
+        schedule=schedule,
+        experiment_vars=experiment_vars,
+    )
+
+    return journal
+
+
+@contextlib.contextmanager
+def reconfigure_chaostoolkit_logger(
+    log_file: Path,
+) -> Generator[logging.Logger, None, None]:
+    ctk_logger = logging.getLogger("logzero_default")
+
+    for handler in list(ctk_logger.handlers):
+        ctk_logger.removeHandler(handler)
+
+    fmt = UTCFormatter(
+        fmt="[%(asctime)s %(levelname)s] [%(module)s:%(lineno)d] %(message)s",
+    )
+
+    handler = logging.FileHandler(log_file.absolute())
+    handler.setLevel(logging.DEBUG)
+    handler.setFormatter(fmt)
+    ctk_logger.addHandler(handler)
+
+    yield ctk_logger
+
+
+class UTCFormatter(logging.Formatter):
+    def formatTime(
+        self, record: logging.LogRecord, datefmt: str | None = None
+    ) -> str:
+        return datetime.fromtimestamp(
+            record.created, tz=timezone.utc
+        ).isoformat()
+
+
+def show_result_url(journal: Journal) -> None:
+    for x in journal.get("experiment", {}).get("extensions", []):
+        if x["name"] == "reliably":
+            url = x.get("execution_url")
+            if url:
+                console.print(f"Check results at {url}")
